<?php
/*
 * netgate_firmware_upgrade.inc
 *
 * part of pfSense (https://www.pfsense.org)
 * Copyright (c) 2017-2023 Rubicon Communications, LLC (Netgate)
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

require_once("globals.inc");
require_once("system.inc");

function check_update($current, $new) {

	# Do not update if we are missing any information.
	if (!isset($current) || !isset($new) ||
	    !isset($current['version']) || !isset($new['version']) ||
	    !isset($new['model'])) {
		return false;
	}

	if ($new['model'] == "4100") {
		if (strlen($current['version']) != 21 ||
		    strlen($new['version']) != 21 ||
		    strpos($current['version'], '-2Ct-uc-') != 11 ||
		    strpos($new['version'], '-2Ct-uc-') != 11) {
			return false;
		}
		$curver = str_replace(array(".", "-2Ct-uc-"), "", $current['version']);
		$newver = str_replace(array(".", "-2Ct-uc-"), "", $new['version']);
	} elseif ($new['model'] == "6100" ||
		  $new['model'] == "6200" ||
		  $new['model'] == "8200" ) {
		if (strlen($current['version']) != 18 ||
		    strlen($new['version']) != 18 ||
		    strpos($current['version'], 't-uc-') != 11 ||
		    strpos($new['version'], 't-uc-') != 11) {
			return false;
		}
		$curver = str_replace(array(".", "t-uc-"), "", $current['version']);
		$newver = str_replace(array(".", "t-uc-"), "", $new['version']);
	} else {
		return ($current['version'] != $new['version']);
	}

	return (intval($newver) > intval($curver));
}

function get_firmware_model() {
	$specplatform = system_identify_specific_platform();

	switch($specplatform['name']) {
	case '4100':
	case '6100':
	case '6200':
	case '7100':
	case '8200':
		return $specplatform['name'];
		break;
	case 'RCC-DFF':
	case 'SG-2220':
		return 'DFF2';
		break;
	case 'SG-2440':
	case 'SG-4860':
	case 'SG-8860':
	case 'RCC-VE':
		return 'RCCVE';
		break;
	case 'RCC':
		return 'RCC';
		break;
	default:
		return '';
	}
}

function is_netgate_hw() {
	return !empty(get_firmware_model());
}

function get_firmware_details($firmware) {
	$details = array();

	# Netgate 6100/6200/8200
	if (preg_match('/^(CORDOBA)-([\d.]{11}\w-uc-\d*)/', $firmware,
	    $matches) && isset($matches[2])) {
		$details['model'] = get_firmware_model();
		$details['version'] = $matches[2];
		return $details;
	}

	# Netgate 4100
	if (preg_match('/^(CORDOBA)-([\d.]{11}-\w{3}-uc-\d*)/', $firmware,
	    $matches) && isset($matches[2])) {
		$details['model'] = get_firmware_model();
		$details['version'] = $matches[2];
		return $details;
	}

	# Silicom/ADI
	if (preg_match('/^ADI_(RCC|RCCVE|DFF2|PLCC)-([\d.]{11})/',
	    $firmware, $matches) && isset($matches[2])) {
		$details['model'] = get_firmware_model();
		$details['version'] = $matches[2];
		return $details;
	}

	return $details;
}

function get_current_firmware_details() {
	$firmware = exec("/bin/kenv -q smbios.bios.version 2>/dev/null");

	$uc_version = get_single_sysctl('dev.cordbuc.0.version');
	if (!empty($uc_version)) {
		$firmware .= "-uc-" . $uc_version;
	}

	return get_firmware_details($firmware);
}

function get_new_firmware_details() {
	$firmware_path = "/usr/local/share/pfSense-pkg-Netgate_Firmware_Upgrade/roms/";

	$rom_names = array(
	    '4100' => 'CORDOBA-03.00.00.01-2Ct-uc-15.fd',
	    '6100' => 'CORDOBA-03.00.00.03t-uc-18.fd',
	    '6200' => 'CORDOBA-03.00.00.03t-uc-18.fd',
	    '7100' => 'ADI_PLCC-01.00.00.20.bin',
	    '8200' => 'CORDOBA-03.00.00.03t-uc-18.fd',
	    'RCC' => 'ADI_RCC-01.00.00.18-nodebug-16m.rom',
	    'RCCVE' => 'ADI_RCCVE-01.00.00.17-nodebug.rom',
	    'DFF2' => 'ADI_DFF2-01.00.00.17-nodebug.rom',
	);

	$model = get_firmware_model();
	$details = array();
	$path = $firmware_path . $rom_names[$model];

	if (empty($model) || empty($rom_names[$model]) ||
	    !file_exists($firmware_path . $rom_names[$model])) {
		return $details;
	}

	$details = get_firmware_details($rom_names[$model]);
	$details['rom_path'] = $firmware_path . $rom_names[$model];

	return $details;
}

function upgrade_firmware($firmware, &$output) {
	global $g;

	putenv("PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:" .
	    "/usr/local/sbin");

	if ($firmware['model'] == '4100' ||
	    $firmware['model'] == '6100' ||
	    $firmware['model'] == '6200' ||
	    $firmware['model'] == '8200') {
		$_gc = exec("/usr/local/sbin/blinkboot-upgrade.sh " .
		    $firmware['rom_path'] . ' 2>&1', $output, $rc);
	} elseif ($firmware['model'] == '7100') {
		$_gc = exec("/usr/local/sbin/adi_smbios_util -w -u " .
		    $firmware['rom_path'] . ' 2>&1', $output, $rc);
	} else {
		$_gc = exec("/usr/local/sbin/adi_flash_util -u " .
		    $firmware['rom_path'] . ' 2>&1', $output, $rc);
	}

	$output = implode("\n", $output);

	@file_put_contents("{$g['conf_path']}/netgate_firmware_upgrade.log",
	    $output);

	return ($rc == 0);
}
